@@ -4,8 +4,8 @@ from urllib.parse import quote_from_bytes, unquote_to_bytes
 
 from .connection import Connection
 from .constants import STATS_KEYS, DEFAULT_BUFFER_SIZE, Step, Namespace, Platform, FESL_DETAILS, LookupType, \
-    STATS_BUFFER_SIZE, DEFAULT_LEADERBOARD_KEYS, LEADERBOARD_BUFFER_SIZE
-from .exceptions import PyBfbc2StatsParameterError, PyBfbc2StatsError, PyBfbc2StatsNotFoundError
+    DEFAULT_LEADERBOARD_KEYS
+from .exceptions import PyBfbc2StatsParameterError, PyBfbc2StatsError, PyBfbc2StatsNotFoundError, PyBfbc2SearchError
 
 
 class Client:
@@ -83,10 +83,9 @@ class Client:
 
         lookup_packet = self.build_user_lookup_packet(identifiers, namespace, lookup_type)
         self.connection.write(lookup_packet)
-        response = self.connection.read()
-        body = response[12:-1]
 
-        return self.parse_list_response(body, b'userInfo.')
+        parsed_response, *_ = self.get_list_response(b'userInfo.')
+        return parsed_response
 
     def lookup_user_identifier(self, identifier: str, namespace: Namespace,  lookup_type: LookupType) -> dict:
         results = self.lookup_user_identifiers([identifier], namespace, lookup_type)
@@ -96,6 +95,16 @@ class Client:
 
         return results.pop()
 
+    def search_name(self, screen_name: str) -> dict:
+        if self.track_steps and Step.login not in self.complete_steps:
+            self.login()
+
+        search_packet = self.build_search_packet(screen_name)
+        self.connection.write(search_packet)
+
+        parsed_response, metadata = self.get_list_response(b'users.')
+        return self.format_search_response(parsed_response, metadata)
+
     def get_stats(self, userid: int, keys: List[bytes] = STATS_KEYS) -> dict:
         if self.track_steps and Step.login not in self.complete_steps:
             self.login()
@@ -105,7 +114,7 @@ class Client:
         for chunk_packet in chunk_packets:
             self.connection.write(chunk_packet)
 
-        parsed_response = self.get_stats_response(STATS_BUFFER_SIZE, b'stats.')
+        parsed_response, *_ = self.get_list_response(b'stats.')
         return self.dict_list_to_dict(parsed_response)
 
     def get_leaderboard(self, min_rank: int = 1, max_rank: int = 50, sort_by: bytes = b'score',
@@ -116,16 +125,16 @@ class Client:
         leaderboard_packet = self.build_leaderboard_query_packet(min_rank, max_rank, sort_by, keys)
         self.connection.write(leaderboard_packet)
 
-        parsed_response = self.get_stats_response(LEADERBOARD_BUFFER_SIZE, b'stats.')
+        parsed_response, *_ = self.get_list_response(b'stats.')
         # Turn sub lists into dicts and return result
         return [{key: Client.dict_list_to_dict(value) if isinstance(value, list) else value
                  for (key, value) in persona.items()} for persona in parsed_response]
 
-    def get_stats_response(self, buffer_size: int, list_parse_prefix: bytes) -> List[dict]:
+    def get_list_response(self, list_parse_prefix: bytes) -> Tuple[List[dict], List[bytes]]:
         response = b''
         last_packet = False
         while not last_packet:
-            packet = self.connection.read(buffer_size)
+            packet = self.connection.read()
             data, last_packet = self.handle_stats_response_packet(packet, list_parse_prefix + b'[]=')
             response += data
 
@@ -201,6 +210,13 @@ class Client:
 
         return lookup_packet
 
+    @staticmethod
+    def build_search_packet(screen_name: str) -> bytes:
+        return Client.build_packet(
+            b'acct\xc0\x00\x00\x1c',
+            b'TXN=NuSearchOwners\nscreenName=' + screen_name.encode('utf8') + b'\nsearchType=1\nretrieveUserIds=0'
+        )
+
     @staticmethod
     def build_leaderboard_query_packet(min_rank: int, max_rank: int, sort_by: bytes, keys: List[bytes]) -> bytes:
         key_list = Client.build_list_body(keys, b'keys')
@@ -239,7 +255,7 @@ class Client:
         return chunk_packets
 
     @staticmethod
-    def parse_list_response(raw_response: bytes, entry_prefix: bytes) -> List[dict]:
+    def parse_list_response(raw_response: bytes, entry_prefix: bytes) -> Tuple[List[dict], List[bytes]]:
         lines = raw_response.split(b'\n')
         # Assign lines to either data or meta lines
         meta_lines = []
@@ -281,7 +297,7 @@ class Client:
                 # Add scaler value to dict
                 datasets[index][key] = value
 
-        return datasets
+        return datasets, meta_lines
 
     @staticmethod
     def handle_stats_response_packet(packet: bytes, length_indicator: bytes) -> Tuple[bytes, bool]:
@@ -290,10 +306,15 @@ class Client:
 
         # Check for errors
         if b'errorCode' in body:
+            method_line = next((line for line in lines if line.startswith(b'TXN')), b'')
+            method = method_line.split(b'=').pop()
             error_code_line = next((line for line in lines if line.startswith(b'errorCode=')), b'')
             error_code = error_code_line.split(b'=').pop()
             if error_code == b'21':
                 raise PyBfbc2StatsParameterError('FESL returned invalid parameter error')
+            elif error_code == b'104' and method == b'NuSearchOwners':
+                # Error code is returned if a) no results matched the query or b) too many results matched the query
+                raise PyBfbc2SearchError('FESL found no or too many results matching the search query')
             else:
                 raise PyBfbc2StatsError(f'FESL returned an error (code {error_code.decode("utf")})')
         elif b'data=' not in body and length_indicator not in body:
@@ -320,3 +341,13 @@ class Client:
     def dict_list_to_dict(dict_list: List[dict]) -> dict:
         sorted_list = sorted(dict_list, key=lambda x: x['key'])
         return {entry['key']: entry['value'] for entry in sorted_list}
+
+    @staticmethod
+    def format_search_response(parsed_response: List[dict], metadata: List[bytes]) -> dict:
+        namespace_line = next(line for line in metadata if line.startswith(b'nameSpaceId'))
+        namespace = namespace_line.split(b'=').pop()
+
+        return {
+            'namespace': namespace.decode('utf8'),
+            'users': parsed_response
+        }
